searchData={"items":[{"type":"module","title":"LangChain.Chains.DataExtractionChain","doc":"Defines an LLMChain for performing data extraction from a body of text.\n\nProvide the schema for desired information to be parsed into. It is treated as\nthough there are 0 to many instances of the data structure being described so\ninformation is returned as an array.\n\nOriginally based on:\n- https://github.com/langchain-ai/langchainjs/blob/main/langchain/src/chains/openai_functions/extraction.ts#L43","ref":"LangChain.Chains.DataExtractionChain.html"},{"type":"module","title":"Example - LangChain.Chains.DataExtractionChain","doc":"# JSONSchema definition of data we want to capture or extract.\n    schema_parameters = %{\n      type: \"object\",\n      properties: %{\n        person_name: %{type: \"string\"},\n        person_age: %{type: \"number\"},\n        person_hair_color: %{type: \"string\"},\n        dog_name: %{type: \"string\"},\n        dog_breed: %{type: \"string\"}\n      },\n      required: []\n    }\n\n    # Model setup\n    {:ok, chat} = ChatOpenAI.new(%{temperature: 0})\n\n    # run the chain on the text information\n    data_prompt =\n      \"Alex is 5 feet tall. Claudia is 4 feet taller than Alex and jumps higher than him.\n      Claudia is a brunette and Alex is blonde. Alex's dog Frosty is a labrador and likes to play hide and seek.\"\n\n    {:ok, result} = LangChain.Chains.DataExtractionChain.run(chat, schema_parameters, data_prompt)\n\n    # Example result\n    [\n      %{\n        \"dog_breed\" => \"labrador\",\n        \"dog_name\" => \"Frosty\",\n        \"person_age\" => nil,\n        \"person_hair_color\" => \"blonde\",\n        \"person_name\" => \"Alex\"\n      },\n      %{\n        \"dog_breed\" => nil,\n        \"dog_name\" => nil,\n        \"person_age\" => nil,\n        \"person_hair_color\" => \"brunette\",\n        \"person_name\" => \"Claudia\"\n      }\n    ]\n\nThe `schema_parameters` in the previous example can also be expressed using a\nlist of `LangChain.FunctionParam` structs. An equivalent version looks like\nthis:\n\n    alias LangChain.FunctionParam\n\n    schema_parameters = [\n      FunctionParam.new!(%{name: \"person_name\", type: :string}),\n      FunctionParam.new!(%{name: \"person_age\", type: :number}),\n      FunctionParam.new!(%{name: \"person_hair_color\", type: :string}),\n      FunctionParam.new!(%{name: \"dog_name\", type: :string}),\n      FunctionParam.new!(%{name: \"dog_breed\", type: :string})\n    ]\n    |> FunctionParam.to_parameters_schema()","ref":"LangChain.Chains.DataExtractionChain.html#module-example"},{"type":"function","title":"LangChain.Chains.DataExtractionChain.build_extract_function/1","doc":"Build the function to expose to the LLM that can be called for data\nextraction.","ref":"LangChain.Chains.DataExtractionChain.html#build_extract_function/1"},{"type":"function","title":"LangChain.Chains.DataExtractionChain.run/4","doc":"Run the data extraction chain.","ref":"LangChain.Chains.DataExtractionChain.html#run/4"},{"type":"module","title":"LangChain.Chains.LLMChain","doc":"","ref":"LangChain.Chains.LLMChain.html"},{"type":"function","title":"LangChain.Chains.LLMChain.__struct__/0","doc":"Define an LLMChain. This is the heart of the LangChain library.\n\nThe chain deals with functions, a function map, delta tracking, last_message\ntracking, conversation messages, and verbose logging. This helps by separating\nthese responsibilities from the LLM making it easier to support additional\nLLMs because the focus is on communication and formats instead of all the\nextra logic.","ref":"LangChain.Chains.LLMChain.html#__struct__/0"},{"type":"function","title":"LangChain.Chains.LLMChain.add_functions/2","doc":"Add more functions to an LLMChain.","ref":"LangChain.Chains.LLMChain.html#add_functions/2"},{"type":"function","title":"LangChain.Chains.LLMChain.add_message/2","doc":"Add a received Message struct to the chain. The LLMChain tracks the\n`last_message` received and the complete list of messages exchanged. Depending\non the message role, the chain may be in a pending or incomplete state where\na response from the LLM is anticipated.","ref":"LangChain.Chains.LLMChain.html#add_message/2"},{"type":"function","title":"LangChain.Chains.LLMChain.add_messages/2","doc":"Add a set of Message structs to the chain. This enables quickly building a chain\nfor submitting to an LLM.","ref":"LangChain.Chains.LLMChain.html#add_messages/2"},{"type":"function","title":"LangChain.Chains.LLMChain.apply_delta/2","doc":"Apply a received MessageDelta struct to the chain. The LLMChain tracks the\ncurrent merged MessageDelta state. When the final delta is received that\ncompletes the message, the LLMChain is updated to clear the `delta` and the\n`last_message` and list of messages are updated.","ref":"LangChain.Chains.LLMChain.html#apply_delta/2"},{"type":"function","title":"LangChain.Chains.LLMChain.apply_deltas/2","doc":"Apply a list of deltas to the chain.","ref":"LangChain.Chains.LLMChain.html#apply_deltas/2"},{"type":"function","title":"LangChain.Chains.LLMChain.apply_prompt_templates/3","doc":"Apply a set of PromptTemplates to the chain. The list of templates can also\ninclude Messages with no templates. Provide the inputs to apply to the\ntemplates for rendering as a message. The prepared messages are applied to the\nchain.","ref":"LangChain.Chains.LLMChain.html#apply_prompt_templates/3"},{"type":"function","title":"LangChain.Chains.LLMChain.cancel_delta/2","doc":"Remove an incomplete MessageDelta from `delta` and add a Message with the\ndesired status to the chain.","ref":"LangChain.Chains.LLMChain.html#cancel_delta/2"},{"type":"function","title":"LangChain.Chains.LLMChain.common_validation/1","doc":"","ref":"LangChain.Chains.LLMChain.html#common_validation/1"},{"type":"function","title":"LangChain.Chains.LLMChain.delta_to_message/1","doc":"Convert any hanging delta of the chain to a message and append to the chain.\n\nIf the delta is `nil`, the chain is returned unmodified.","ref":"LangChain.Chains.LLMChain.html#delta_to_message/1"},{"type":"function","title":"LangChain.Chains.LLMChain.execute_function/2","doc":"If the `last_message` is a `%Message{role: :function_call}`, then the linked\nfunction is executed. If there is no `last_message` or the `last_message` is\nnot a `:function_call`, the LLMChain is returned with no action performed.\nThis makes it safe to call any time.\n\nThe `context` is additional data that will be passed to the executed function.\nThe value given here will override any `custom_context` set on the LLMChain.\nIf not set, the global `custom_context` is used.\n\nhttps://platform.openai.com/docs/guides/gpt/function-calling","ref":"LangChain.Chains.LLMChain.html#execute_function/2"},{"type":"function","title":"LangChain.Chains.LLMChain.new/1","doc":"Start a new LLMChain configuration.\n\n    {:ok, chain} = LLMChain.new(%{\n      llm: %ChatOpenAI{model: \"gpt-3.5-turbo\", stream: true},\n      messages: [%Message.new_system!(\"You are a helpful assistant.\")]\n    })","ref":"LangChain.Chains.LLMChain.html#new/1"},{"type":"function","title":"LangChain.Chains.LLMChain.new!/1","doc":"Start a new LLMChain configuration and return it or raise an error if invalid.\n\n    chain = LLMChain.new!(%{\n      llm: %ChatOpenAI{model: \"gpt-3.5-turbo\", stream: true},\n      messages: [%Message.new_system!(\"You are a helpful assistant.\")]\n    })","ref":"LangChain.Chains.LLMChain.html#new!/1"},{"type":"function","title":"LangChain.Chains.LLMChain.quick_prompt/2","doc":"Convenience function for setting the prompt text for the LLMChain using\nprepared text.","ref":"LangChain.Chains.LLMChain.html#quick_prompt/2"},{"type":"function","title":"LangChain.Chains.LLMChain.run/2","doc":"Run the chain on the LLM using messages and any registered functions. This\nformats the request for a ChatLLMChain where messages are passed to the API.\n\nWhen successful, it returns `{:ok, updated_chain, message_or_messages}`","ref":"LangChain.Chains.LLMChain.html#run/2"},{"type":"function","title":"Options - LangChain.Chains.LLMChain.run/2","doc":"- `:while_needs_response` - repeatedly evaluates functions and submits to the\n  LLM so long as we still expect to get a response.\n- `:callback_fn` - the callback function to execute as messages are received.","ref":"LangChain.Chains.LLMChain.html#run/2-options"},{"type":"function","title":"LangChain.Chains.LLMChain.update_custom_context/3","doc":"Update the LLMChain's `custom_context` map. Passing in a `context_update` map\nwill by default merge the map into the existing `custom_context`.\n\nUse the `:as` option to:\n- `:merge` - Merge update changes in. Default.\n- `:replace` - Replace the context with the `context_update`.","ref":"LangChain.Chains.LLMChain.html#update_custom_context/3"},{"type":"type","title":"LangChain.Chains.LLMChain.t/0","doc":"","ref":"LangChain.Chains.LLMChain.html#t:t/0"},{"type":"module","title":"LangChain.Chains.RoutingChain","doc":"Run a router based on a user's initial prompt to determine what category best\nmatches from the given options. If there is no good match, the value \"DEFAULT\"\nis returned.\n\nHere's an example:\n\n    routes = [\n      PromptRoute.new!(%{\n        name: \"marketing_email\",\n        description: \"Create a marketing focused email\",\n        chain: marketing_email_chain\n      }),\n      PromptRoute.new!(%{\n        name: \"blog_post\",\n        description: \"Create a blog post that will be linked from the company's landing page\",\n        chain: blog_post_chain\n      }),\n    ]\n\n    selected_route =\n      RoutingChain.new(%{\n        llm: ChatOpenAI.new(%{model: \"gpt-3.5-turbo\", stream: false}),\n        input_text: \"Let's create a marketing blog post about our new product 'Fuzzy Furries'\",\n        routes: routes,\n        default_route: PromptRoute.new!(%{name: \"DEFAULT\", chain: fallback_chain})\n      })\n      |> RoutingChain.evaluate()\n\n    # The PromptRoute for the `blog_post` should be returned as the `selected_route`.\n\nThe `llm` is the model used to make the determination of which route is the\nbest match. A smaller, faster LLM may be a great choice for the routing\ndecision, then a more complex LLM may be used for a selected route.\n\nThe `default_route` is required and is used as a fallback if the user's prompt\ndoesn't match any of the specified routes. It may also be used in some\nfallback error situations as well.","ref":"LangChain.Chains.RoutingChain.html"},{"type":"function","title":"LangChain.Chains.RoutingChain.evaluate/2","doc":"Runs the RoutingChain and evaluates the result to return the selected chain.","ref":"LangChain.Chains.RoutingChain.html#evaluate/2"},{"type":"function","title":"LangChain.Chains.RoutingChain.new/1","doc":"Start a new RoutingChain.","ref":"LangChain.Chains.RoutingChain.html#new/1"},{"type":"function","title":"LangChain.Chains.RoutingChain.new!/1","doc":"Start a new RoutingChain and return it or raise an error if invalid.","ref":"LangChain.Chains.RoutingChain.html#new!/1"},{"type":"function","title":"LangChain.Chains.RoutingChain.run/2","doc":"Run a simple RoutingChain to analyze the input_text and determine which of the\ngiven routes is the best match.\n\nA simpler, faster LLM may be a great fit for running the analysis. If it fails\nto find a good match, the `default_route` is used. The `default_route`'s name\nis supplied to the LLM as well. The name \"DEFAULT\" is suggested for this\nroute.","ref":"LangChain.Chains.RoutingChain.html#run/2"},{"type":"type","title":"LangChain.Chains.RoutingChain.t/0","doc":"","ref":"LangChain.Chains.RoutingChain.html#t:t/0"},{"type":"module","title":"LangChain.Chains.TextToTitleChain","doc":"A convenience chain for turning a user's prompt text into a summarized title\nfor the anticipated conversation.","ref":"LangChain.Chains.TextToTitleChain.html"},{"type":"function","title":"LangChain.Chains.TextToTitleChain.evaluate/2","doc":"Runs the TextToTitleChain and evaluates the result to return the final answer.\nIf it was unable to generate a title, the `fallback_title` is returned.","ref":"LangChain.Chains.TextToTitleChain.html#evaluate/2"},{"type":"function","title":"LangChain.Chains.TextToTitleChain.new/1","doc":"Start a new LLMChain configuration.\n\n    {:ok, chain} = LLMChain.new(%{\n      llm: %ChatOpenAI{model: \"gpt-3.5-turbo\", stream: false},\n      input_text: \"Let's create a marketing blog post about our new product 'Fuzzy Furries'\"\n    })","ref":"LangChain.Chains.TextToTitleChain.html#new/1"},{"type":"function","title":"LangChain.Chains.TextToTitleChain.new!/1","doc":"Start a new TextToTitleChain and return it or raise an error if invalid.\n\n    chain = TextToTitleChain.new!(%{\n      llm: %ChatOpenAI{model: \"gpt-3.5-turbo\", stream: false},\n      input_text: \"Let's create a marketing blog post about our new product 'Fuzzy Furries'\"\n    })","ref":"LangChain.Chains.TextToTitleChain.html#new!/1"},{"type":"function","title":"LangChain.Chains.TextToTitleChain.run/2","doc":"Run a simple LLMChain to summarize the user's prompt into a title for the\nconversation. Uses the provided model. Recommend faster, simpler LLMs without\nstreaming.\n\nIf it fails to summarize to a title, it returns the default text.\n\n    new_title = TextToTitleChain.new!(%{\n      llm: %ChatOpenAI{model: \"gpt-3.5-turbo\", stream: false},\n      input_text: \"Let's create a marketing blog post about our new product 'Fuzzy Furries'\"\n    })\n    |> TextToTitleChain.run()","ref":"LangChain.Chains.TextToTitleChain.html#run/2"},{"type":"type","title":"LangChain.Chains.TextToTitleChain.t/0","doc":"","ref":"LangChain.Chains.TextToTitleChain.html#t:t/0"},{"type":"module","title":"LangChain.ChatModels.ChatGoogleAI","doc":"Parses and validates inputs for making a request for the Google AI  Chat API.\n\nConverts response into more specialized `LangChain` data structures.","ref":"LangChain.ChatModels.ChatGoogleAI.html"},{"type":"function","title":"LangChain.ChatModels.ChatGoogleAI.call/4","doc":"Calls the Google AI API passing the ChatGoogleAI struct with configuration, plus\neither a simple message or the list of messages to act as the prompt.\n\nOptionally pass in a list of functions available to the LLM for requesting\nexecution in response.\n\nOptionally pass in a callback function that can be executed as data is\nreceived from the API.\n\n**NOTE:** This function *can* be used directly, but the primary interface\nshould be through `LangChain.Chains.LLMChain`. The `ChatGoogleAI` module is more focused on\ntranslating the `LangChain` data structures to and from the OpenAI API.\n\nAnother benefit of using `LangChain.Chains.LLMChain` is that it combines the\nstorage of messages, adding functions, adding custom context that should be\npassed to functions, and automatically applying `LangChain.MessageDelta`\nstructs as they are are received, then converting those to the full\n`LangChain.Message` once fully complete.","ref":"LangChain.ChatModels.ChatGoogleAI.html#call/4"},{"type":"function","title":"LangChain.ChatModels.ChatGoogleAI.complete_final_delta/1","doc":"","ref":"LangChain.ChatModels.ChatGoogleAI.html#complete_final_delta/1"},{"type":"function","title":"LangChain.ChatModels.ChatGoogleAI.do_process_response/2","doc":"","ref":"LangChain.ChatModels.ChatGoogleAI.html#do_process_response/2"},{"type":"function","title":"LangChain.ChatModels.ChatGoogleAI.for_api/3","doc":"","ref":"LangChain.ChatModels.ChatGoogleAI.html#for_api/3"},{"type":"function","title":"LangChain.ChatModels.ChatGoogleAI.new/1","doc":"Setup a ChatGoogleAI client configuration.","ref":"LangChain.ChatModels.ChatGoogleAI.html#new/1"},{"type":"function","title":"LangChain.ChatModels.ChatGoogleAI.new!/1","doc":"Setup a ChatGoogleAI client configuration and return it or raise an error if invalid.","ref":"LangChain.ChatModels.ChatGoogleAI.html#new!/1"},{"type":"type","title":"LangChain.ChatModels.ChatGoogleAI.t/0","doc":"","ref":"LangChain.ChatModels.ChatGoogleAI.html#t:t/0"},{"type":"behaviour","title":"LangChain.ChatModels.ChatModel","doc":"","ref":"LangChain.ChatModels.ChatModel.html"},{"type":"callback","title":"LangChain.ChatModels.ChatModel.call/4","doc":"","ref":"LangChain.ChatModels.ChatModel.html#c:call/4"},{"type":"type","title":"LangChain.ChatModels.ChatModel.call_response/0","doc":"","ref":"LangChain.ChatModels.ChatModel.html#t:call_response/0"},{"type":"type","title":"LangChain.ChatModels.ChatModel.t/0","doc":"","ref":"LangChain.ChatModels.ChatModel.html#t:t/0"},{"type":"module","title":"LangChain.ChatModels.ChatOllamaAI","doc":"Represents the [Ollama AI Chat model](https://github.com/jmorganca/ollama/blob/main/docs/api.md#generate-a-chat-completion)\n\nParses and validates inputs for making a requests from the Ollama Chat API.\n\nConverts responses into more specialized `LangChain` data structures.\n\nThe module's functionalities include:\n\n- Initializing a new `ChatOllamaAI` struct with defaults or specific attributes.\n- Validating and casting input data to fit the expected schema.\n- Preparing and sending requests to the Ollama AI service API.\n- Managing both streaming and non-streaming API responses.\n- Processing API responses to convert them into suitable message formats.\n\nThe `ChatOllamaAI` struct has fields to configure the AI, including but not limited to:\n\n- `endpoint`: URL of the Ollama AI service.\n- `model`: The AI model used, e.g., \"llama2:latest\".\n- `receive_timeout`: Max wait time for AI service responses.\n- `temperature`: Influences the AI's response creativity.\n\nFor detailed info on on all other parameters see documentation here:\nhttps://github.com/jmorganca/ollama/blob/main/docs/modelfile.md#valid-parameters-and-values\n\nThis module is for use within LangChain and follows the `ChatModel` behavior,\noutlining callbacks AI chat models must implement.\n\nUsage examples and more details are in the LangChain documentation or the\nmodule's function docs.","ref":"LangChain.ChatModels.ChatOllamaAI.html"},{"type":"function","title":"LangChain.ChatModels.ChatOllamaAI.call/4","doc":"Calls the Ollama Chat Completion API struct with configuration, plus\neither a simple message or the list of messages to act as the prompt.\n\n**NOTE:** This API as of right now does not support functions. More\ninformation here: https://github.com/jmorganca/ollama/issues/1729\n\n**NOTE:** This function *can* be used directly, but the primary interface\nshould be through `LangChain.Chains.LLMChain`. The `ChatOllamaAI` module is more focused on\ntranslating the `LangChain` data structures to and from the Ollama API.\n\nAnother benefit of using `LangChain.Chains.LLMChain` is that it combines the\nstorage of messages, adding functions, adding custom context that should be\npassed to functions, and automatically applying `LangChain.MessageDelta`\nstructs as they are are received, then converting those to the full\n`LangChain.Message` once fully complete.","ref":"LangChain.ChatModels.ChatOllamaAI.html#call/4"},{"type":"function","title":"LangChain.ChatModels.ChatOllamaAI.do_process_response/1","doc":"","ref":"LangChain.ChatModels.ChatOllamaAI.html#do_process_response/1"},{"type":"function","title":"LangChain.ChatModels.ChatOllamaAI.for_api/3","doc":"Return the params formatted for an API request.","ref":"LangChain.ChatModels.ChatOllamaAI.html#for_api/3"},{"type":"function","title":"LangChain.ChatModels.ChatOllamaAI.new/1","doc":"Creates a new `ChatOllamaAI` struct with the given attributes.","ref":"LangChain.ChatModels.ChatOllamaAI.html#new/1"},{"type":"function","title":"LangChain.ChatModels.ChatOllamaAI.new!/1","doc":"Creates a new `ChatOllamaAI` struct with the given attributes. Will raise an error if the changeset is invalid.","ref":"LangChain.ChatModels.ChatOllamaAI.html#new!/1"},{"type":"type","title":"LangChain.ChatModels.ChatOllamaAI.t/0","doc":"","ref":"LangChain.ChatModels.ChatOllamaAI.html#t:t/0"},{"type":"module","title":"LangChain.ChatModels.ChatOpenAI","doc":"Represents the [OpenAI ChatModel](https://platform.openai.com/docs/api-reference/chat/create).\n\nParses and validates inputs for making a requests from the OpenAI Chat API.\n\nConverts responses into more specialized `LangChain` data structures.\n\n- https://github.com/openai/openai-cookbook/blob/main/examples/How_to_call_functions_with_chat_models.ipynb","ref":"LangChain.ChatModels.ChatOpenAI.html"},{"type":"function","title":"LangChain.ChatModels.ChatOpenAI.call/4","doc":"Calls the OpenAI API passing the ChatOpenAI struct with configuration, plus\neither a simple message or the list of messages to act as the prompt.\n\nOptionally pass in a list of functions available to the LLM for requesting\nexecution in response.\n\nOptionally pass in a callback function that can be executed as data is\nreceived from the API.\n\n**NOTE:** This function *can* be used directly, but the primary interface\nshould be through `LangChain.Chains.LLMChain`. The `ChatOpenAI` module is more focused on\ntranslating the `LangChain` data structures to and from the OpenAI API.\n\nAnother benefit of using `LangChain.Chains.LLMChain` is that it combines the\nstorage of messages, adding functions, adding custom context that should be\npassed to functions, and automatically applying `LangChain.MessageDelta`\nstructs as they are are received, then converting those to the full\n`LangChain.Message` once fully complete.","ref":"LangChain.ChatModels.ChatOpenAI.html#call/4"},{"type":"function","title":"LangChain.ChatModels.ChatOpenAI.for_api/3","doc":"Return the params formatted for an API request.","ref":"LangChain.ChatModels.ChatOpenAI.html#for_api/3"},{"type":"function","title":"LangChain.ChatModels.ChatOpenAI.new/1","doc":"Setup a ChatOpenAI client configuration.","ref":"LangChain.ChatModels.ChatOpenAI.html#new/1"},{"type":"function","title":"LangChain.ChatModels.ChatOpenAI.new!/1","doc":"Setup a ChatOpenAI client configuration and return it or raise an error if invalid.","ref":"LangChain.ChatModels.ChatOpenAI.html#new!/1"},{"type":"type","title":"LangChain.ChatModels.ChatOpenAI.t/0","doc":"","ref":"LangChain.ChatModels.ChatOpenAI.html#t:t/0"},{"type":"module","title":"LangChain.Config","doc":"Utility that handles interaction with the application's configuration.","ref":"LangChain.Config.html"},{"type":"function","title":"LangChain.Config.resolve/2","doc":"Resolves the given key from the application's configuration returning the\nwrapped expanded value. If the value is a function it gets evaluated, if the\nvalue is a tuple of three elements it gets applied.","ref":"LangChain.Config.html#resolve/2"},{"type":"protocol","title":"LangChain.ForOpenAIApi","doc":"A protocol that defines a way for converting the LangChain Elixir data structs\nto an OpenAI supported data structure and format for making an API call.","ref":"LangChain.ForOpenAIApi.html"},{"type":"function","title":"LangChain.ForOpenAIApi.for_api/1","doc":"Protocol callback function for converting different structs into a form that\ncan be passed to the OpenAI API.","ref":"LangChain.ForOpenAIApi.html#for_api/1"},{"type":"type","title":"LangChain.ForOpenAIApi.t/0","doc":"All the types that implement this protocol.","ref":"LangChain.ForOpenAIApi.html#t:t/0"},{"type":"module","title":"LangChain.Function","doc":"Defines a \"function\" that can be provided to an LLM for the LLM to optionally\nexecute and pass argument data to.\n\nA function is defined using a schema.\n\n* `name` - The name of the function given to the LLM.\n* `description` - A description of the function provided to the LLM. This\n  should describe what the function is used for or what it returns. This\n  information is used by the LLM to decide which function to call and for what\n  purpose.\n* ` parameters` - A list of `Function.FunctionParam` structs that are\n  converted to a JSONSchema format. (Use in place of `parameters_schema`)\n* ` parameters_schema` - A [JSONSchema\n  structure](https://json-schema.org/learn/getting-started-step-by-step.html)\n  that describes the required data structure format for how arguments are\n  passed to the function. (Use if greater control or unsupported features are\n  needed.)\n* `function` - An Elixir function to execute when an LLM requests to execute\n  the function.\n\nWhen passing arguments from an LLM to a function, they go through a single\n`map` argument. This allows for multiple keys or named parameters.","ref":"LangChain.Function.html"},{"type":"module","title":"Example - LangChain.Function","doc":"This example defines a function that an LLM can execute for performing basic\nmath calculations. **NOTE:** This is a partial implementation of the\n`LangChain.Tools.Calculator`.\n\n    Function.new(%{\n      name: \"calculator\",\n      description: \"Perform basic math calculations\",\n      parameters_schema: %{\n        type: \"object\",\n        properties: %{\n          expression: %{type: \"string\", description: \"A simple mathematical expression.\"}\n        },\n        required: [\"expression\"]\n      },\n      function:\n        fn(%{\"expression\" => expr} = _args, _context) ->\n          \"Uh... I don't know!\"\n        end)\n    })\n\nThe `function` attribute is an Elixir function that can be executed when the\nfunction is \"called\" by the LLM.\n\nThe `args` argument is the JSON data passed by the LLM after being parsed to a\nmap.\n\nThe `context` argument is passed through as the `context` on a\n`LangChain.Chains.LLMChain`. This is whatever context data is needed for the\nfunction to do it's work.\n\nContext examples may be user_id, account_id, account struct, billing level,\netc.","ref":"LangChain.Function.html#module-example"},{"type":"module","title":"Function Parameters - LangChain.Function","doc":"The `parameters` field is a list of `LangChain.FunctionParam` structs. This is\na convenience for defining the parameters to the function. If it does not work\nfor more complex use-cases, then use the `parameters_schema` to declare it as\nneeded.\n\nThe `parameters_schema` is an Elixir map that follows a\n[JSONSchema](https://json-schema.org/learn/getting-started-step-by-step.html)\nstructure. It is used to define the required data structure format for\nreceiving data to the function from the LLM.\n\nNOTE: Only use `parameters` or `parameters_schema`, not both.","ref":"LangChain.Function.html#module-function-parameters"},{"type":"module","title":"Expanded Parameter Examples - LangChain.Function","doc":"Function with no arguments:\n\n    alias LangChain.Function\n\n    Function.new!(%{name: \"get_current_user_info\"})\n\nFunction that takes a simple required argument:\n\n    alias LangChain.FunctionParam\n\n    Function.new!(%{name: \"set_user_name\", parameters: [\n      FunctionParam.new!(%{name: \"user_name\", type: :string, required: true})\n    ]})\n\nFunction that takes an array of strings:\n\n    Function.new!(%{name: \"set_tags\", parameters: [\n      FunctionParam.new!(%{name: \"tags\", type: :array, item_type: \"string\"})\n    ]})\n\nFunction that takes two arguments and one is an object/map:\n\n    Function.new!(%{name: \"update_preferences\", parameters: [\n      FunctionParam.new!(%{name: \"unique_code\", type: :string, required: true})\n      FunctionParam.new!(%{name: \"data\", type: :object, object_properties: [\n        FunctionParam.new!(%{name: \"auto_complete_email\", type: :boolean}),\n        FunctionParam.new!(%{name: \"items_per_page\", type: :integer}),\n      ]})\n    ]})\n\nThe `LangChain.FunctionParam` is nestable allowing for arrays of object and\nobjects with nested objects.","ref":"LangChain.Function.html#module-expanded-parameter-examples"},{"type":"function","title":"LangChain.Function.execute/3","doc":"Execute the function passing in arguments and additional optional context.\nThis is called by a `LangChain.Chains.LLMChain` when a `Function` execution is\nrequested by the LLM.","ref":"LangChain.Function.html#execute/3"},{"type":"function","title":"LangChain.Function.get_display_text/3","doc":"Given a list of functions, return the `display_text` for the named function.\nIf it not found, return the fallback text.","ref":"LangChain.Function.html#get_display_text/3"},{"type":"function","title":"LangChain.Function.new/1","doc":"Build a new function.","ref":"LangChain.Function.html#new/1"},{"type":"function","title":"LangChain.Function.new!/1","doc":"Build a new function and return it or raise an error if invalid.","ref":"LangChain.Function.html#new!/1"},{"type":"type","title":"LangChain.Function.t/0","doc":"","ref":"LangChain.Function.html#t:t/0"},{"type":"module","title":"LangChain.FunctionParam","doc":"Define a function parameter as a struct. Used to generate the expected\nJSONSchema data for describing one or more arguments being passed to a\n`LangChain.Function`.\n\nNote: This is not intended to be a fully compliant implementation of\n[JSONSchema\ntypes](https://json-schema.org/understanding-json-schema/reference/type). This\nis intended to be a convenience for working with the most common situations\nwhen working with an LLM that understands JSONSchema.\n\nSupports:\n\n* simple values - string, integer, number, boolean\n* enum values - `enum: [\"alpha\", \"beta\"]`. The values can be strings,\n  integers, etc.\n* array values - `type: :array` couples with `item_type: \"string\"` to express\n  it is an array of.\n  * `item_type` is optional. When omitted, it can be a mixed array.\n  * `item_type: \"object\"` allows for creating an array of objects. Use\n    `object_properties: [...]` to describe the structure of the objects.\n* objects - Define the object's expected values or supported structure using\n  `object_properties`.\n\nThe function `to_parameters_schema/1` is used to convert a list of\n`FunctionParam` structs into a JSONSchema formatted data map.","ref":"LangChain.FunctionParam.html"},{"type":"function","title":"LangChain.FunctionParam.new/1","doc":"Build a new FunctionParam struct.","ref":"LangChain.FunctionParam.html#new/1"},{"type":"function","title":"LangChain.FunctionParam.new!/1","doc":"Build a new `FunctionParam` struct and return it or raise an error if invalid.","ref":"LangChain.FunctionParam.html#new!/1"},{"type":"function","title":"LangChain.FunctionParam.required_properties/1","doc":"Return the list of required property names.","ref":"LangChain.FunctionParam.html#required_properties/1"},{"type":"function","title":"LangChain.FunctionParam.to_json_schema/2","doc":"Transform a `FunctionParam` to a JSONSchema compatible definition that is\nadded to the passed in `data` map.","ref":"LangChain.FunctionParam.html#to_json_schema/2"},{"type":"function","title":"LangChain.FunctionParam.to_parameters_schema/1","doc":"Transform a list of `FunctionParam` structs into a map expressing the structure\nin a JSONSchema compatible way.","ref":"LangChain.FunctionParam.html#to_parameters_schema/1"},{"type":"type","title":"LangChain.FunctionParam.t/0","doc":"","ref":"LangChain.FunctionParam.html#t:t/0"},{"type":"module","title":"LangChain.Gettext","doc":"A module providing Internationalization with a gettext-based API.\n\nBy using [Gettext](https://hexdocs.pm/gettext),\nyour module gains a set of macros for translations, for example:\n\n    import LangChain.Gettext\n\n    # Simple translation\n    gettext(\"Here is the string to translate\")\n\n    # Plural translation\n    ngettext(\"Here is the string to translate\",\n             \"Here are the strings to translate\",\n             3)\n\n    # Domain-based translation\n    dgettext(\"errors\", \"Here is the error message to translate\")\n\nSee the [Gettext Docs](https://hexdocs.pm/gettext) for detailed usage.","ref":"LangChain.Gettext.html"},{"type":"macro","title":"LangChain.Gettext.dgettext/3","doc":"","ref":"LangChain.Gettext.html#dgettext/3"},{"type":"macro","title":"LangChain.Gettext.dgettext_noop/2","doc":"","ref":"LangChain.Gettext.html#dgettext_noop/2"},{"type":"macro","title":"LangChain.Gettext.dngettext/5","doc":"","ref":"LangChain.Gettext.html#dngettext/5"},{"type":"macro","title":"LangChain.Gettext.dngettext_noop/3","doc":"","ref":"LangChain.Gettext.html#dngettext_noop/3"},{"type":"macro","title":"LangChain.Gettext.dpgettext/4","doc":"","ref":"LangChain.Gettext.html#dpgettext/4"},{"type":"macro","title":"LangChain.Gettext.dpgettext_noop/3","doc":"","ref":"LangChain.Gettext.html#dpgettext_noop/3"},{"type":"macro","title":"LangChain.Gettext.dpngettext/6","doc":"","ref":"LangChain.Gettext.html#dpngettext/6"},{"type":"macro","title":"LangChain.Gettext.dpngettext_noop/4","doc":"","ref":"LangChain.Gettext.html#dpngettext_noop/4"},{"type":"macro","title":"LangChain.Gettext.gettext/2","doc":"","ref":"LangChain.Gettext.html#gettext/2"},{"type":"macro","title":"LangChain.Gettext.gettext_comment/1","doc":"","ref":"LangChain.Gettext.html#gettext_comment/1"},{"type":"macro","title":"LangChain.Gettext.gettext_noop/1","doc":"","ref":"LangChain.Gettext.html#gettext_noop/1"},{"type":"function","title":"LangChain.Gettext.handle_missing_bindings/2","doc":"","ref":"LangChain.Gettext.html#handle_missing_bindings/2"},{"type":"function","title":"LangChain.Gettext.handle_missing_plural_translation/7","doc":"","ref":"LangChain.Gettext.html#handle_missing_plural_translation/7"},{"type":"function","title":"LangChain.Gettext.handle_missing_translation/5","doc":"","ref":"LangChain.Gettext.html#handle_missing_translation/5"},{"type":"function","title":"LangChain.Gettext.lgettext/5","doc":"","ref":"LangChain.Gettext.html#lgettext/5"},{"type":"function","title":"LangChain.Gettext.lngettext/7","doc":"","ref":"LangChain.Gettext.html#lngettext/7"},{"type":"macro","title":"LangChain.Gettext.ngettext/4","doc":"","ref":"LangChain.Gettext.html#ngettext/4"},{"type":"macro","title":"LangChain.Gettext.ngettext_noop/2","doc":"","ref":"LangChain.Gettext.html#ngettext_noop/2"},{"type":"macro","title":"LangChain.Gettext.pgettext/3","doc":"","ref":"LangChain.Gettext.html#pgettext/3"},{"type":"macro","title":"LangChain.Gettext.pgettext_noop/2","doc":"","ref":"LangChain.Gettext.html#pgettext_noop/2"},{"type":"macro","title":"LangChain.Gettext.pngettext/5","doc":"","ref":"LangChain.Gettext.html#pngettext/5"},{"type":"macro","title":"LangChain.Gettext.pngettext_noop/3","doc":"","ref":"LangChain.Gettext.html#pngettext_noop/3"},{"type":"module","title":"LangChain.Message","doc":"Models a complete `Message` for a chat LLM.","ref":"LangChain.Message.html"},{"type":"module","title":"Roles - LangChain.Message","doc":"- `:system` - a system message. Typically just one and it occurs first as a\n  primer for how the LLM should behave.\n\n- `:user` - The user or application responses. Typically represents the\n  \"human\" element of the exchange.\n\n- `:assistant` - Responses coming back from the LLM.\n\n- `:arguments` - The `arguments` can be set as a map where each key is an\n  \"argument\". If set as a String, it is expected to be a JSON formatted string\n  and will be parsed to a map. If there is an error parsing the arguments to\n  JSON, it is considered an error.\n\n  An empty map `%{}` means no arguments are passed.\n\n- `:function` - A message for returning the result of executing a\n  `function_call`.","ref":"LangChain.Message.html#module-roles"},{"type":"module","title":"Functions - LangChain.Message","doc":"A `function_call` comes from the `:assistant` role. The `function_name`\nidentifies the named function to execute.\n\nCreate a message of role `:function` to provide the function response.","ref":"LangChain.Message.html#module-functions"},{"type":"function","title":"LangChain.Message.is_function_call?/1","doc":"Return if a Message is a function_call.","ref":"LangChain.Message.html#is_function_call?/1"},{"type":"function","title":"LangChain.Message.new/1","doc":"Build a new message and return an `:ok`/`:error` tuple with the result.","ref":"LangChain.Message.html#new/1"},{"type":"function","title":"LangChain.Message.new!/1","doc":"Build a new message and return it or raise an error if invalid.","ref":"LangChain.Message.html#new!/1"},{"type":"function","title":"LangChain.Message.new_assistant/2","doc":"Create a new assistant message which represents a response from the AI or LLM.","ref":"LangChain.Message.html#new_assistant/2"},{"type":"function","title":"LangChain.Message.new_assistant!/2","doc":"Create a new assistant message which represents a response from the AI or LLM.","ref":"LangChain.Message.html#new_assistant!/2"},{"type":"function","title":"LangChain.Message.new_function/2","doc":"Create a new function message to represent the result of an executed\nfunction.","ref":"LangChain.Message.html#new_function/2"},{"type":"function","title":"LangChain.Message.new_function!/2","doc":"Create a new function message to represent the result of an executed\nfunction.","ref":"LangChain.Message.html#new_function!/2"},{"type":"function","title":"LangChain.Message.new_function_call/2","doc":"Create a new function_call message to represent the request for a function to\nbe executed.","ref":"LangChain.Message.html#new_function_call/2"},{"type":"function","title":"LangChain.Message.new_function_call!/2","doc":"Create a new function_call message to represent the request for a function to\nbe executed.","ref":"LangChain.Message.html#new_function_call!/2"},{"type":"function","title":"LangChain.Message.new_system/1","doc":"Create a new system message which can prime the AI/Assistant for how to\nrespond.","ref":"LangChain.Message.html#new_system/1"},{"type":"function","title":"LangChain.Message.new_system!/1","doc":"Create a new system message which can prime the AI/Assistant for how to\nrespond.","ref":"LangChain.Message.html#new_system!/1"},{"type":"function","title":"LangChain.Message.new_user/1","doc":"Create a new user message which represents a human message or a message from\nthe application.","ref":"LangChain.Message.html#new_user/1"},{"type":"function","title":"LangChain.Message.new_user!/1","doc":"Create a new user message which represents a human message or a message from\nthe application.","ref":"LangChain.Message.html#new_user!/1"},{"type":"type","title":"LangChain.Message.status/0","doc":"","ref":"LangChain.Message.html#t:status/0"},{"type":"type","title":"LangChain.Message.t/0","doc":"","ref":"LangChain.Message.html#t:t/0"},{"type":"module","title":"LangChain.MessageDelta","doc":"Models a \"delta\" message from a chat LLM. A delta is a small chunk, or piece\nof a much larger complete message. A series of deltas can are used to\nconstruct a complete message.\n\nDelta messages must be applied in order for them to be valid. Delta messages\ncan be combined and transformed into a `LangChain.Message` once the final\npiece is received.","ref":"LangChain.MessageDelta.html"},{"type":"module","title":"Roles - LangChain.MessageDelta","doc":"* `:unknown` - The role data is missing for the delta.\n* `:assistant` - Responses coming back from the LLM.","ref":"LangChain.MessageDelta.html#module-roles"},{"type":"module","title":"Function calling - LangChain.MessageDelta","doc":"* `:function_name` - A message from the LLM expressing the intent to execute a\n  function that was previously declared available to it.\n\n  The `arguments` will eventually be parsed from JSON. However, as deltas are\n  streamed, the arguments come in as text. Once it is _fully received_ it can\n  be parsed as JSON, but it cannot be used before it is complete as it will\n  not be valid JSON.","ref":"LangChain.MessageDelta.html#module-function-calling"},{"type":"function","title":"LangChain.MessageDelta.merge_delta/2","doc":"Merge two `MessageDelta` structs. The first `MessageDelta` is the `primary`\none that smaller deltas are merged into.\n\n    iex> delta_1 =\n    ...>   %LangChain.MessageDelta{\n    ...>     content: nil,\n    ...>     index: 0,\n    ...>     function_name: nil,\n    ...>     role: :assistant,\n    ...>     arguments: nil,\n    ...>     status: :incomplete\n    ...>   }\n    iex> delta_2 =\n    ...>   %LangChain.MessageDelta{\n    ...>     content: \"Hello\",\n    ...>     index: 0,\n    ...>     function_name: nil,\n    ...>     role: :unknown,\n    ...>     arguments: nil,\n    ...>     status: :incomplete\n    ...>   }\n    iex> LangChain.MessageDelta.merge_delta(delta_1, delta_2)\n    %LangChain.MessageDelta{content: \"Hello\", status: :incomplete, index: 0, function_name: nil, role: :assistant, arguments: nil}\n\nA set of deltas can be easily merged like this:\n\n    [first | rest] = list_of_delta_message\n\n    Enum.reduce(rest, first, fn new_delta, acc ->\n      MessageDelta.merge_delta(acc, new_delta)\n    end)","ref":"LangChain.MessageDelta.html#merge_delta/2"},{"type":"function","title":"LangChain.MessageDelta.new/1","doc":"Create a new `MessageDelta` that represents a message chunk.","ref":"LangChain.MessageDelta.html#new/1"},{"type":"function","title":"LangChain.MessageDelta.new!/1","doc":"Create a new `MessageDelta` that represents a message chunk and return it or\nraise an error if invalid.","ref":"LangChain.MessageDelta.html#new!/1"},{"type":"function","title":"LangChain.MessageDelta.to_message/1","doc":"Convert the MessageDelta to a Message. Can only convert a fully complete\nMessageDelta.\n\nThis is assumed to be the result of merging all the received `MessageDelta`s.\nAn error is returned if the `status` is `:incomplete`.\n\nIf the `MessageDelta` fails to convert to a `LangChain.Message`, an error is\nreturned with the reason.","ref":"LangChain.MessageDelta.html#to_message/1"},{"type":"type","title":"LangChain.MessageDelta.t/0","doc":"","ref":"LangChain.MessageDelta.html#t:t/0"},{"type":"module","title":"LangChain.PromptTemplate","doc":"Enables defining a prompt, optionally as a template, but delaying the final\nbuilding of it until a later time when input values are substituted in.\n\nThis also supports the ability to create a Message from a PromptTemplate.\n\nAn LLM conversation is made up of a set of messages. PromptTemplates are a\ntool to help build messages.\n\n    # Create a template and convert it to a message\n    prompt = PromptTemplate.new!(%{text: \"My template\", role: :user})\n    %LangChain.Message{} = message = PromptTemplate.to_message(prompt)\n\nPromptTemplates are powerful because they support Elixir's EEx templates\nallowing for parameter substitution. This is helpful when we want to prepare a\ntemplate message and plan to later substitute in information from the user.\n\nHere's an example of setting up a template using a parameter then later\nproviding the input value.\n\n    prompt = PromptTemplate.from_template!(\"What's a name for a company that makes <%= @product %>?\")\n\n    # later, format the final text after after applying the values.\n    PromptTemplate.format(prompt, %{product: \"colorful socks\"})\n    #=> \"What's a name for a company that makes colorful socks?\"","ref":"LangChain.PromptTemplate.html"},{"type":"function","title":"LangChain.PromptTemplate.format/2","doc":"Format the prompt template with inputs to replace with assigns. It returns the\nformatted text.\n\n    prompt = PromptTemplate.from_template!(\"Suggest a good name for a company that makes <%= @product %>?\")\n    PromptTemplate.format(prompt, %{product: \"colorful socks\"})\n    #=> \"Suggest a good name for a company that makes colorful socks?\"\n\nA PromptTemplate supports storing input values on the struct. These could be\nset when the template is defined. If an input value is not provided when the\n`format` function is called, any inputs on the struct will be used.","ref":"LangChain.PromptTemplate.html#format/2"},{"type":"function","title":"LangChain.PromptTemplate.format_composed/3","doc":"Formats a PromptTemplate at two levels. Supports providing a list of\n`composed_of` templates that are all combined into a `full_template`.\n\nFor this example, we'll use an overall template layout like this:\n\n    full_prompt =\n      PromptTemplate.from_template!(~s(<%= @introduction %>\n\n      <%= @example %>\n\n      <%= @start %>))\n\nThis template is made up of 3 more specific templates. Let's start with the\nintroduction sub-template.\n\n    introduction_prompt =\n      PromptTemplate.from_template!(\"You are impersonating <%= @person %>.\")\n\nThe `introduction` takes a parameter for which person it should impersonate.\nThe desired person is not provided here and will come in later.\n\nLet's next look at the `example` prompt:\n\n    example_prompt =\n      PromptTemplate.from_template!(~s(Here's an example of an interaction:\n        Q: <%= @example_q %>\n        A: <%= @example_a %>))\n\nThis defines a sample interaction for the LLM as a model of what we're looking\nfor. Primarily, this template is used to define the pattern we want to use for\nthe interaction. Again, this template takes parameters for the sample question\nand answer.\n\nFinally, there is the `start` section of the overall prompt. In this example,\nthat might be a question presented by a user asking a question of our\nimpersonating AI.\n\n    start_prompt =\n      PromptTemplate.from_template!(~s(Now, do this for real!\n      Q: <%= @input %>\n      A:))\n\nWe have the overall template defined and templates that define each of the\nsmaller portions. The `format_composed` function let's us combine those all\ntogether and build the complete text to pass to an LLM.\n\n      formatted_prompt =\n        PromptTemplate.format_composed(\n          full_prompt,\n          %{\n            introduction: introduction_prompt,\n            example: example_prompt,\n            start: start_prompt\n          },\n          %{\n            person: \"Elon Musk\",\n            example_q: \"What's your favorite car?\",\n            example_a: \"Tesla\",\n            input: \"What's your favorite social media site?\"\n          }\n        )\n\nWe provide the PromptTemplate for the overall prompt, then provide the inputs,\nwhich are themselves prompts.\n\nFinally, we provide a map of values for all the parameters that still need\nvalues. For this example, this is what the final prompt looks like that is\npresented to the LLM.\n\n    ~s(You are impersonating Elon Musk.\n\n    Here's an example of an interaction:\n    Q: What's your favorite car?\n    A: Tesla\n\n    Now, do this for real!\n    Q: What's your favorite social media site?\n    A:)\n\nUsing a setup like this, we can easily swap out who we are impersonating and\nallow the user to interact with that persona.\n\nWith everything defined, this is all it takes to now talk with an Abraham\nLincoln impersonation:\n\n    formatted_prompt =\n      PromptTemplate.format_composed(\n        full_prompt,\n        %{\n          introduction: introduction_prompt,\n          example: example_prompt,\n          start: start_prompt\n        },\n        %{\n          person: \"Abraham Lincoln\",\n          example_q: \"What is your nickname?\",\n          example_a: \"Honest Abe\",\n          input: \"What is one of your favorite pastimes?\"\n        }\n      )","ref":"LangChain.PromptTemplate.html#format_composed/3"},{"type":"function","title":"LangChain.PromptTemplate.format_text/2","doc":"Format the prompt template with inputs to replace embeds. The final replaced\ntext is returned.\n\nOperates directly on text to apply the inputs. This does not take the\nPromptTemplate struct.\n\n    PromptTemplate.format_text(\"Hi! My name is <%= @name %>.\", %{name: \"Jose\"})\n    #=> \"Hi! My name is Jose.\"","ref":"LangChain.PromptTemplate.html#format_text/2"},{"type":"function","title":"LangChain.PromptTemplate.from_template/1","doc":"Build a PromptTemplate struct from a template string.\n\nShortcut function for building a user prompt.\n\n    {:ok, prompt} = PromptTemplate.from_template(\"Suggest a good name for a company that makes <%= @product %>?\")","ref":"LangChain.PromptTemplate.html#from_template/1"},{"type":"function","title":"LangChain.PromptTemplate.from_template!/1","doc":"Build a PromptTemplate struct from a template string and return the struct or error if invalid.\n\nShortcut function for building a user prompt.\n\n    prompt = PromptTemplate.from_template!(\"Suggest a good name for a company that makes <%= @product %>?\")","ref":"LangChain.PromptTemplate.html#from_template!/1"},{"type":"function","title":"LangChain.PromptTemplate.new/1","doc":"","ref":"LangChain.PromptTemplate.html#new/1"},{"type":"function","title":"LangChain.PromptTemplate.new!/1","doc":"Create a new PromptTemplate struct using the attributes. If invalid, an\nexception is raised with the reason.","ref":"LangChain.PromptTemplate.html#new!/1"},{"type":"function","title":"LangChain.PromptTemplate.to_message/2","doc":"Transform a PromptTemplate to a `LangChain.Message`. Provide the inputs at the time of\ntransformation to render the final content.","ref":"LangChain.PromptTemplate.html#to_message/2"},{"type":"function","title":"LangChain.PromptTemplate.to_message!/2","doc":"Transform a PromptTemplate to a `LangChain.Message`. Provide the inputs at the time of\ntransformation to render the final content. Raises an exception if invalid.","ref":"LangChain.PromptTemplate.html#to_message!/2"},{"type":"function","title":"LangChain.PromptTemplate.to_messages!/2","doc":"Transform a list of PromptTemplates into a list of `LangChain.Message`s.\nApplies the inputs to the list of prompt templates. If any of the prompt\nentries are invalid or fail, an exception is raised.","ref":"LangChain.PromptTemplate.html#to_messages!/2"},{"type":"type","title":"LangChain.PromptTemplate.t/0","doc":"","ref":"LangChain.PromptTemplate.html#t:t/0"},{"type":"module","title":"LangChain.Routing.PromptRoute","doc":"Defines a route or direction a prompting interaction with an LLM can take.\n\nThis helps add complexity and specificity to an LLM's instructions without\nusing many tokens and helps avoid combining many things into a single prompt\nsetup. This works by letting the user's initial prompt define how the next\nprompt is setup. A first pass is run on the prompt to determine which of the\nprovided PromptRoutes is the best match.\n\nTo better understand, let's see example of taking the user's input and\nclassifying what supported activity or specialty it is best matched with.\n\nUser prompt:\n> Let's create a marketing focused blog post comparing three types of our\n> trailer hitch products.\n\nWe want our assistant to use different prompts for the different types of\nactivities it's capable of doing. Let's define the activities like this:\n\n    [\n      PromptRoute.new!(%{\n        name: \"marketing_email\",\n        description: \"Create a marketing focused email\",\n        chain: marketing_email_chain\n      }),\n      PromptRoute.new!(%{\n        name: \"blog_post\",\n        description: \"Create a blog post that will be linked from the company's landing page\",\n        chain: blog_post_chain\n      }),\n      PromptRoute.new!(%{\n        name: \"support_triage\",\n        description: \"Triage a customer support request for severity and requested resolution\",\n        chain: support_triage_chain\n      })\n    ]\n\nGiven the user's prompt and the routes we provided, it will choose\n\"blog_post\". This leads to an LLMChain being setup for that purpose where the\nuser's initial prompt can be re-run against the prompts we define for that\nroute.\n\nWhen a `chain` is linked to each route, we can specify the model to use, the\nprompt templates we want, and associate functions that can be used to\naccomplish the task.\n\nThe selected LLMChain can then be run through a UI for chat interactions where\nthe user and AI work together.\n\nThe `name` will be returned by the LLM when a particular route is selected.\n\nThe `description` is given to the LLM to help it determine if the route should\nbe selected based on the current input prompt.","ref":"LangChain.Routing.PromptRoute.html"},{"type":"function","title":"LangChain.Routing.PromptRoute.get_selected/2","doc":"Return the selected route based on the route name.","ref":"LangChain.Routing.PromptRoute.html#get_selected/2"},{"type":"function","title":"LangChain.Routing.PromptRoute.new/1","doc":"Build a new PromptRoute struct.","ref":"LangChain.Routing.PromptRoute.html#new/1"},{"type":"function","title":"LangChain.Routing.PromptRoute.new!/1","doc":"Build a new `PromptRoute` struct and return it or raise an error if invalid.","ref":"LangChain.Routing.PromptRoute.html#new!/1"},{"type":"type","title":"LangChain.Routing.PromptRoute.t/0","doc":"","ref":"LangChain.Routing.PromptRoute.html#t:t/0"},{"type":"module","title":"LangChain.Tools.Calculator","doc":"Defines a Calculator tool for performing basic math calculations.\n\nThis is an example of a pre-built `LangChain.Function` that is designed and\nconfigured for a specific purpose.\n\nThis defines a function to expose to an LLM and provides an implementation for\nthe `execute/2` function for evaluating when an LLM executes the function.\n\nWhen using the `Calculator` tool, you will either need to:\n\n* make repeated calls to run the chain as the tool is called and the results\nare then made available to the LLM before it returns the final result.\n* OR run the chain using the `while_needs_response: true` option like this:\n  `LangChain.LLMChain.run(chain, while_needs_response: true)`","ref":"LangChain.Tools.Calculator.html"},{"type":"module","title":"Example - LangChain.Tools.Calculator","doc":"The following is an example that uses a prompt where math is needed. What\nfollows is the verbose log output.\n\n    {:ok, updated_chain, %Message{} = message} =\n      %{llm: ChatOpenAI.new!(%{temperature: 0}), verbose: true}\n      |> LLMChain.new!()\n      |> LLMChain.add_message(\n        Message.new_user!(\"Answer the following math question: What is 100 + 300 - 200?\")\n      )\n      |> LLMChain.add_functions(Calculator.new!())\n      |> LLMChain.run(while_needs_response: true)\n\nVerbose log output:\n\n    LLM: %LangChain.ChatModels.ChatOpenAI{\n      endpoint: \"https://api.openai.com/v1/chat/completions\",\n      model: \"gpt-3.5-turbo\",\n      temperature: 0.0,\n      frequency_penalty: 0.0,\n      receive_timeout: 60000,\n      n: 1,\n      stream: false\n    }\n    MESSAGES: [\n      %LangChain.Message{\n        content: \"Answer the following math question: What is 100 + 300 - 200?\",\n        index: nil,\n        status: :complete,\n        role: :user,\n        function_name: nil,\n        arguments: nil\n      }\n    ]\n    FUNCTIONS: [\n      %LangChain.Function{\n        name: \"calculator\",\n        description: \"Perform basic math calculations\",\n        function: #Function<0.108164323/2 in LangChain.Tools.Calculator.execute>,\n        parameters_schema: %{\n          properties: %{\n            expression: %{\n              description: \"A simple mathematical expression.\",\n              type: \"string\"\n            }\n          },\n          required: [\"expression\"],\n          type: \"object\"\n        }\n      }\n    ]\n    SINGLE MESSAGE RESPONSE: %LangChain.Message{\n      content: nil,\n      index: 0,\n      status: :complete,\n      role: :assistant,\n      function_name: \"calculator\",\n      arguments: %{\"expression\" => \"100 + 300 - 200\"}\n    }\n    EXECUTING FUNCTION: \"calculator\"\n    FUNCTION RESULT: \"200\"\n    SINGLE MESSAGE RESPONSE: %LangChain.Message{\n      content: \"The answer to the math question \"What is 100 + 300 - 200?\" is 200.\",\n      index: 0,\n      status: :complete,\n      role: :assistant,\n      function_name: nil,\n      arguments: nil\n    }","ref":"LangChain.Tools.Calculator.html#module-example"},{"type":"function","title":"LangChain.Tools.Calculator.execute/2","doc":"Performs the calculation specified in the expression and returns the response\nto be used by the the LLM.","ref":"LangChain.Tools.Calculator.html#execute/2"},{"type":"function","title":"LangChain.Tools.Calculator.new/0","doc":"Define the \"calculator\" function. Returns a success/failure response.","ref":"LangChain.Tools.Calculator.html#new/0"},{"type":"function","title":"LangChain.Tools.Calculator.new!/0","doc":"Define the \"calculator\" function. Raises an exception if function creation fails.","ref":"LangChain.Tools.Calculator.html#new!/0"},{"type":"module","title":"LangChain.Utils","doc":"Collection of helpful utilities mostly for internal use.","ref":"LangChain.Utils.html"},{"type":"function","title":"LangChain.Utils.changeset_error_to_string/1","doc":"Return changeset errors as text with comma separated description.","ref":"LangChain.Utils.html#changeset_error_to_string/1"},{"type":"function","title":"LangChain.Utils.conditionally_add_to_map/3","doc":"Only add the key to the map if the value is present. When the value is a list,\nthe key will not be added when the list is empty. If the value is `nil`, it\nwill not be added.","ref":"LangChain.Utils.html#conditionally_add_to_map/3"},{"type":"function","title":"LangChain.Utils.fire_callback/3","doc":"Fire a streaming callback if present.","ref":"LangChain.Utils.html#fire_callback/3"},{"type":"function","title":"LangChain.Utils.handle_stream_fn/3","doc":"Create a function to handle the streaming request.","ref":"LangChain.Utils.html#handle_stream_fn/3"},{"type":"function","title":"LangChain.Utils.translate_error/1","doc":"Translates an error message using gettext.","ref":"LangChain.Utils.html#translate_error/1"},{"type":"function","title":"LangChain.Utils.translate_errors/2","doc":"Translates the errors for a field from a keyword list of errors.","ref":"LangChain.Utils.html#translate_errors/2"},{"type":"function","title":"LangChain.Utils.validate_llm_is_struct/1","doc":"Validation helper. Validates a struct changeset that the LLM is a struct.","ref":"LangChain.Utils.html#validate_llm_is_struct/1"},{"type":"type","title":"LangChain.Utils.callback_data/0","doc":"","ref":"LangChain.Utils.html#t:callback_data/0"},{"type":"module","title":"LangChain.Utils.ApiOverride","doc":"Tools for overriding API results. Used for testing.\n\nWorks by setting and checking for special use of the Process dictionary.","ref":"LangChain.Utils.ApiOverride.html"},{"type":"module","title":"Test Example - LangChain.Utils.ApiOverride","doc":"import LangChain.Utils.ApiOverride\n\n    model = ChatOpenAI.new!(%{temperature: 1, stream: true})\n\n    # Define the fake response to return\n    fake_messages = [\n      [MessageDelta.new!(%{role: :assistant, content: nil, status: :incomplete})],\n      [MessageDelta.new!(%{content: \"Sock\", status: :incomplete})]\n    ]\n\n    # Made NOT LIVE here. Will not make the external call to the LLM\n    set_api_override({:ok, fake_messages})\n\n    # We can construct an LLMChain from a PromptTemplate and an LLM.\n    {:ok, updated_chain, _response} =\n      %{llm: model, verbose: false}\n      |> LLMChain.new!()\n      |> LLMChain.add_message(\n        Message.new_user!(\"What is a good name for a company that makes colorful socks?\")\n      )\n      |> LLMChain.run()\n\n    assert %Message{role: :assistant, content: \"Sock\"} = updated_chain.last_message","ref":"LangChain.Utils.ApiOverride.html#module-test-example"},{"type":"function","title":"LangChain.Utils.ApiOverride.get_api_override/0","doc":"Get the API override to return. Returned as `{:ok, response}`. If not set, it\nreturns `:not_set`.","ref":"LangChain.Utils.ApiOverride.html#get_api_override/0"},{"type":"function","title":"LangChain.Utils.ApiOverride.override_api_return?/0","doc":"Return if an override for the API response is set. Used for testing.","ref":"LangChain.Utils.ApiOverride.html#override_api_return?/0"},{"type":"function","title":"LangChain.Utils.ApiOverride.set_api_override/1","doc":"Set the term to return as a fake API response.","ref":"LangChain.Utils.ApiOverride.html#set_api_override/1"},{"type":"module","title":"LangChain.Utils.ChainResult","doc":"Module to help when working with the results of a chain.","ref":"LangChain.Utils.ChainResult.html"},{"type":"function","title":"LangChain.Utils.ChainResult.to_map/3","doc":"Write the result to the given map as the value of the given key.","ref":"LangChain.Utils.ChainResult.html#to_map/3"},{"type":"function","title":"LangChain.Utils.ChainResult.to_map!/3","doc":"Write the result to the given map as the value of the given key. If invalid,\nan exception is raised.","ref":"LangChain.Utils.ChainResult.html#to_map!/3"},{"type":"function","title":"LangChain.Utils.ChainResult.to_string/1","doc":"Return the result of the chain as a string. Returned in an `:ok` tuple format.\nAn `{:error, reason}` is returned for various reasons. These include:\n- The last message of the chain is not an `:assistant` message.\n- The last message of the chain is incomplete.\n- There is no last message.\n\nThis supports passing in the final, updated LLMChain, or the result of the\n`LLMChain.run/2` function.","ref":"LangChain.Utils.ChainResult.html#to_string/1"},{"type":"function","title":"LangChain.Utils.ChainResult.to_string!/1","doc":"Return the last message's content when it is valid to use it. Otherwise it\nraises and exception with the reason why it cannot be used. See the docs for\n`to_string/2` for details.","ref":"LangChain.Utils.ChainResult.html#to_string!/1"},{"type":"exception","title":"LangChain.LangChainError","doc":"Exception used for raising LangChain specific errors.\n\nIt stores the `:message`. Passing an Ecto.Changeset with an error\nconverts the error into a string message.\n\n    raise LangChainError, changeset\n\n    raise LangChainError, \"Message text\"","ref":"LangChain.LangChainError.html"},{"type":"function","title":"LangChain.LangChainError.exception/1","doc":"Create the exception using either a message or a changeset who's errors are\nconverted to a message.","ref":"LangChain.LangChainError.html#exception/1"},{"type":"type","title":"LangChain.LangChainError.t/0","doc":"","ref":"LangChain.LangChainError.html#t:t/0"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.1.8 (2024-02-16) - Changelog","doc":"**Breaking change**: `RoutingChain`'s required values changed. Previously, `default_chain` was assigned an `%LLMChain{}` to return when no more specific routes matched.\n\nThis was changed to be `default_route`. It now expects a `%PromptRoute{}` to be provided.\n\nHere's how to make the change:\n\n      selected_route =\n        RoutingChain.new(%{\n          llm: ChatOpenAI.new(%{model: \"gpt-3.5-turbo\", stream: false}),\n          input_text: user_input_text,\n          routes: routes,\n          default_route: PromptRoute.new!(%{name: \"DEFAULT\", chain: fallback_chain})\n        })\n        |> RoutingChain.evaluate()\n\nThe `default_chain` was changed to `default_route` and now expects a `PromptRoute` to be provided. The above example includes a sample default route that includes an optional `fallback_chain`.\n\nPreviously, the returned value from `RoutingChain.evaluate/1` was a `selected_chain`; it now returns the `selected_route`.\n\n**Why was this changed?**\n\nThis was changed to make it easier to use a `PromptChain` when there isn't an associated `%LLMChain{}` for it. The application must just need the answer of which route was selected.\n\nThis includes the change of not requiring a `%PromptChain{}`'s `description` or `chain` field.\n\n**Other Changes**\n* Add support for Ollama open source models by @medoror in https://github.com/brainlid/langchain/pull/70\n* Add clause to match call_response spec by @medoror in https://github.com/brainlid/langchain/pull/72\n* Add max_tokens option for OpenAI calls by @matthusby in https://github.com/brainlid/langchain/pull/73","ref":"changelog.html#v0-1-8-2024-02-16"},{"type":"extras","title":"v0.1.7 (2024-01-18) - Changelog","doc":"- Improvements for more intelligent agents - https://github.com/brainlid/langchain/pull/61\n  - adds `LangChain.Chains.RoutingChain` - first-pass LLM chain to select the best route to take given the user's initial prompt\n  - adds `LangChain.Chains.TextToTitleChain` - turn the user's prompt into a title for the conversation\n- Removed the need for a function to send a message to the process for how to display the function being executed\n- Updated dependencies\n- Add support for Google AI / Gemini Pro model by @jadengis in https://github.com/brainlid/langchain/pull/59\n- Built-in automatic retries when underlying Mint connection is closed in https://github.com/brainlid/langchain/pull/68","ref":"changelog.html#v0-1-7-2024-01-18"},{"type":"extras","title":"v0.1.6 (2023-12-12) - Changelog","doc":"- Fix for correct usage of new Req retry setting. PR #57","ref":"changelog.html#v0-1-6-2023-12-12"},{"type":"extras","title":"v0.1.5 (2023-12-11) - Changelog","doc":"- Upgraded Req to v0.4.8. It contains a needed retry fix for certain situations.\n- Fix OpenAI returns \"Unrecognized request argument supplied: api_key\" PR #54","ref":"changelog.html#v0-1-5-2023-12-11"},{"type":"extras","title":"v0.1.4 (2023-12-11) - Changelog","doc":"- Merged PR #45 - https://github.com/brainlid/langchain/pull/45\n  - Added `LangChain.Utils.ChainResult` for helper functions when working with LLMChain result values.\n- Merged PR #46 - https://github.com/brainlid/langchain/pull/46\n  - Add possibility to use api_key per chat invocation.\n- Merged PR #51 - https://github.com/brainlid/langchain/pull/51\n  - Update req 0.4.7\n  - Hopefully resolves issue where Finch connections would be closed and a now does a built-in retry.\n- Merged PR #52 - https://github.com/brainlid/langchain/pull/52\n  - Allow overriding OpenAI compatible API endpoint. Caller can pass an alternate `endpoint`.","ref":"changelog.html#v0-1-4-2023-12-11"},{"type":"extras","title":"v0.1.3 (2023-12-01) - Changelog","doc":"- Merged PR #43 - https://github.com/brainlid/langchain/pull/43\n  - Add Finch retry strategy to OpenAI Chat API requests\n- Merged PR #39 - https://github.com/brainlid/langchain/pull/39\n  - Changed ENV key from `OPENAI_KEY` to `OPENAI_API_KEY` to be consistent with the OpenAI docs.\n- Merged PR #36 - https://github.com/brainlid/langchain/pull/36\n  - Support specifying the `seed` with OpenAI calls. Used in testing for more deterministic behavior.\n- Merged PR #34 - https://github.com/brainlid/langchain/pull/34\n  - Enable sending the `json_response` flag with OpenAI model requests.\n- Created `LangChain.FunctionParam` to express JSONSchema-friendly data structures. Supports basic types, arrays, enums, objects, arrays of objects and nested objects.\n  - Still allows for full control over JSONSchema by providing an override `parameters_schema` object to full self-describe it.","ref":"changelog.html#v0-1-3-2023-12-01"},{"type":"extras","title":"v0.1.2 (2023-10-26) - Changelog","doc":"- refactor(chat_open_ai): Harden `do_process_response` by @Cardosaum in https://github.com/brainlid/langchain/pull/21\n  - Improve JSON error handling result from ChatGPT\n- Update req to 0.4.4 by @medoror in https://github.com/brainlid/langchain/pull/25\n  - Updated to Req 0.4.4","ref":"changelog.html#v0-1-2-2023-10-26"},{"type":"extras","title":"v0.1.1 (2023-10-10) - Changelog","doc":"Minor update release.\n\n- added \"update_custom_context\" to LLMChain\n- added support for setting the OpenAI-Organization header in requests\n- fixed data extraction chain and improved the prompt\n- make chatgpt response tests more robust","ref":"changelog.html#v0-1-1-2023-10-10"},{"type":"extras","title":"v0.1.0 (2023-09-18) - Changelog","doc":"Initial release when published to [hex.pm](https://hex.pm).","ref":"changelog.html#v0-1-0-2023-09-18"},{"type":"extras","title":"Getting Started","doc":"# Getting Started\n\nAfter installing the dependency, let's look at the simplest example to get started.\n\nThis is interactively available as a Livebook notebook named [`getting_started.livemd`](https://github.com/brainlid/langchain/tree/main/notebooks/getting_started.livemd).","ref":"getting_started.html"},{"type":"extras","title":"Basic Example - Getting Started","doc":"Let's build the simplest full LLMChain example so we can see how to make a call to ChatGPT from our Elixir application.\n\n**NOTE:** This assumes your `OPENAI_KEY` is already set as a secret for this notebook.\n\n```elixir\nalias LangChain.Chains.LLMChain\nalias LangChain.ChatModels.ChatOpenAI\nalias LangChain.Message\n\n{:ok, _updated_chain, response} =\n  %{llm: ChatOpenAI.new!(%{model: \"gpt-4\"})}\n  |> LLMChain.new!()\n  |> LLMChain.add_message(Message.new_user!(\"Testing, testing!\"))\n  |> LLMChain.run()\n\nresponse.content\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\"1, 2, 3. Assistant is online and ready to assist you!\"\n```\n\nNice! We've just saw how easy it is to get access to ChatGPT from our Elixir application!\n\nLet's build on that example and define some `system` context for our conversation.","ref":"getting_started.html#basic-example"},{"type":"extras","title":"Adding a System Message - Getting Started","doc":"When working with ChatGPT and other LLMs, the conversation works as a series of messages. The first message is the `system` message. This is used to define the context for the conversation. Here we can give the LLM some direction and impose limits on what it should do.\n\nLet's create a system message followed by a user message.\n\n```elixir\n{:ok, _updated_chain, response} =\n  %{llm: ChatOpenAI.new!(%{model: \"gpt-4\"})}\n  |> LLMChain.new!()\n  |> LLMChain.add_messages([\n    Message.new_system!(\n      \"You are an unhelpful assistant. Do not directly help or assist the user.\"\n    ),\n    Message.new_user!(\"What's the capital of the United States?\")\n  ])\n  |> LLMChain.run()\n\nresponse.content\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\"Why don't you try looking it up online? There's so much information readily available on the internet. You might even learn a few other interesting facts about the country.\"\n```\n\nHere's the answer it gave me when I ran it:\n\n> Why don't you try looking it up online? There's so much information readily available on the internet. You might even learn a few other interesting facts about the country.\n\nWhat I love about this is we can see the power of the `system` message. It completely changed the way the LLM would behave by default.\n\nBeyond the `system` message, we pass back a whole collection of messages as the conversation continues. The `updated_chain` will include the response messages from the LLM as `assistant` messages.","ref":"getting_started.html#adding-a-system-message"},{"type":"extras","title":"Streaming Responses - Getting Started","doc":"If we want to display the messages as they are returned in the teletype way LLMs can, then we want to stream the responses.\n\nIn this example, we'll output the responses as they are streamed back. That happens in a callback function that we provide.\n\n```elixir\nalias LangChain.MessageDelta\n\ncallback = fn\n  %MessageDelta{} = data ->\n    # we received a piece of data\n    IO.write(data.content)\n\n  %Message{} = data ->\n    # we received the finshed message once fully complete\n    IO.puts(\"\")\n    IO.puts(\"\")\n    IO.inspect(data.content, label: \"COMPLETED MESSAGE\")\nend\n\n{:ok, _updated_chain, response} =\n  %{llm: ChatOpenAI.new!(%{model: \"gpt-4\", stream: true})}\n  |> LLMChain.new!()\n  |> LLMChain.add_messages([\n    Message.new_system!(\"You are a helpful assistant.\"),\n    Message.new_user!(\"Write a haiku about the capital of the United States\")\n  ])\n  |> LLMChain.run(callback_fn: callback)\n\nresponse.content\n# streamed\n#==> Washington D.C. stands,\n#... Monuments reflect history,\n#... Power's heart expands.\n\n#==> COMPLETED MESSAGE: \"Washington D.C. stands,\\nMonuments reflect history,\\nPower's heart expands.\"\n```","ref":"getting_started.html#streaming-responses"}],"content_type":"text/markdown"}